@@ -2,8 +2,8 @@
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """Lightweight :term:`computation` graphs for Python."""
 
-__version__ = "8.0.0"
-__release_date__ = "7 May 2020, 02:15"
+__version__ = "8.0.1"
+__release_date__ = "7 May 2020, 02:58"
 __title__ = "graphtik"
 __summary__ = __doc__.splitlines()[0]
 __license__ = "Apache-2.0"
@@ -48,8 +48,7 @@ from .modifiers import (
     vararg,
     varargs,
 )
-from .pipeline import compose
-from .op import NULL_OP, operation
+from .composition import NULL_OP, operation, compose
 
 ## SEE ALSO: `.plot.active_plotter_plugged()`, `.plot.set_active_plotter()` &
 #  `.plot.get_active_plotter()` configs, not imported, unless plot is needed..
