@@ -1,7 +1,7 @@
 # Copyright 2016, Yahoo Inc.
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """
-A :term:`modifier` change :term:`dependency` behavior during :term:`compilation` or :term:`execution`.
+:term:`modifier`\\s change :term:`dependency` behavior during :term:`compilation` & :term:`execution`.
 
 The `needs` and `provides` annotated with *modifiers* designate, for instance,
 :term:`optional <optionals>` function arguments, or "ghost" :term:`sideffects`.
@@ -9,24 +9,62 @@ The `needs` and `provides` annotated with *modifiers* designate, for instance,
 import enum
 from typing import Optional, Tuple, Union
 
+# fmt: off
+#: Arguments-presence patterns for :class:`_Modifier` constructor.
+#: Combinations missing raise errors.
+_modifier_cstor_matrix = {
+# (7, kw, opt, sfxed, sfx): (STR, REPR) OR None
+70000: None,
+71000: (       "%(dep)s",                  "'%(dep)s'(%(kw)s)"           ),
+71100: (       "%(dep)s",                  "'%(dep)s'(?%(kw)s)"          ),
+70200: (       "%(dep)s",                  "'%(dep)s'(*)"                ),
+70300: (       "%(dep)s",                  "'%(dep)s'(+)"                ),
+70010: ( "sfx: '%(dep)s'",            "sfx: '%(dep)s'"                   ),
+70110: ( "sfx: '%(dep)s'",         "sfx(?): '%(dep)s'"                    ),
+70011: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s', %(sfx)s)"         ),
+71011: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(%(kw)s), %(sfx)s)" ),
+71111: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(?%(kw)s), %(sfx)s)"),
+70211: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(*), %(sfx)s)"      ),
+70311: ("sfxed('%(dep)s', %(sfx)s)", "sfxed('%(dep)s'(+), %(sfx)s)"      ),
+}
+# fmt: on
+
+
+def _match_modifier_args(name, *args):
+    flags = [int(bool(i)) for i in args]
+
+    # expand optional
+    if args[1]:
+        flags[1] = args[1].value
+    pattern = "".join(str(i) for i in flags)
+    pattern = int(f"7{pattern}")
+    if pattern not in _modifier_cstor_matrix:
+        raise ValueError(f"Invalid modifier arguments: {name}, {args}, {pattern}")
+
+    return _modifier_cstor_matrix[pattern]
+
 
 class _Optionals(enum.Enum):
-    optional = 1
+    keyword = 1
     vararg = 2
     varargs = 3
 
-    @property
-    def varargish(self):
-        """True if the :term:`optionals` is a :term:`varargish`. """
-        return self.value > 1
-
 
 class _Modifier(str):
     """
     Annotate a :term:`dependency` with a combination of :term:`modifier`.
 
-    Private, in the sense that users should use the factory functions :func:`.mapped`,
-    :func:`optional` etc, and :func:`is_optional()` predicates.
+    It is private, in the sense that users should use only:
+
+    - the factory functions :func:`.mapped`, :func:`optional` etc,
+    - the predicates :func:`is_optional()`, :func:`is_pure_sfx()` predicates, etc,
+    - and the :func:`dep_rename()`, :func:`dep_strip()` conversion functions
+
+    respectively.
+
+    .. Note::
+        User code better call :func:`_modifier()` factory which may return a plain string
+        if no other arg but ``name`` are given.
     """
 
     # avoid __dict__ on instances
@@ -34,182 +72,130 @@ class _Modifier(str):
         "fn_kwarg",
         "optional",
         "sideffected",
-        "sideffects",
+        "sfx_list",
         "_repr",
     )
 
     #: Map my name in `needs` into this kw-argument of the function.
+    #: :func:`is_mapped()` returns it.
     fn_kwarg: str
     #: required is None, regular optional or varargish?
+    #: :func:`is_optional()` returns it.
+    #: All regulars are `mapped`.
     optional: _Optionals
-    #: An existing `dependency` in `solution` that sustain (must have sustained)
-    #: the :term:`sideffects` by(for) the underlying function.
+    #: the pure-sideffect string or the existing :term:`sideffected` dependency
     sideffected: str
     #: At least one name(s) denoting the :term:`sideffects` modification(s) on
     #: the :term:`sideffected`, performed/required by the operation.
-    #: If it is an empty tuple`, it is an abstract sideffect.
-    sideffects: Tuple[Union[str, None]]
+    #:
+    #: - If it is an empty tuple`, it is an abstract sideffect,
+    #:    and :func:`is_pure_optional()` returns True.
+    #: - If not empty :func:`is_sfxed()` returns true
+    #:   (the :attr:`sideffected`).
+    sfx_list: Tuple[Union[str, None]]
     #: pre-calculated representation
     _repr: str
 
     def __new__(
-        cls,
-        name: str,
-        fn_kwarg: str = None,
-        optional: _Optionals = None,
-        sideffects: Tuple[Union[str, None]] = None,
+        cls, name, fn_kwarg, optional: _Optionals, sideffected, sfx_list, _repr,
     ) -> "_Modifier":
-        sideffected = _repr = None
-        ## Sanity checks and decide
-        #  - string-name on sideffects and
-        #  - repr for all
+        """Warning, returns None! """
+        ## sanity checks & preprocessing
         #
-        assert not optional or _Optionals(optional), ("Invalid optional: ", locals())
-        if optional and optional.varargish:
-            assert not fn_kwarg, (
-                "Varargish cannot map `fn_kwargs` or sideffects:",
-                locals(),
+        if optional is not None and not isinstance(optional, _Optionals):
+            raise ValueError(
+                f"Invalid _Optional enum {optional!r}\n  locals={locals()}"
+            )
+        if sideffected and is_sfx(sideffected):
+            raise ValueError(
+                f"`sideffected` cannot be sideffect, got {sideffected!r}"
+                f"\n  locals={locals()}"
+            )
+        double_sideffects = [
+            f"{type(i).__name__}({i!r})" for i in sfx_list if is_sfx(i)
+        ]
+        if double_sideffects:
+            raise ValueError(
+                f"`sfx_list` cannot contain sideffects, got {double_sideffects!r}"
+                f"\n  locals={locals()}"
             )
-            _repr = f"{optional.name}({str(name)!r})"
-        else:
-            if sideffects is not None:
-                if sideffects == ():
-                    assert fn_kwarg is None, (
-                        "Pure sideffects cannot map `fn_kwarg`:",
-                        locals(),
-                    )
 
-                    # Repr display also optionality (irrelevant to object's identity)
-                    qmark = "?" if optional else ""
-                    _repr = f"sideffect{qmark}: {str(name)!r}"
-                    name = f"sideffect: {str(name)!r}"
-                else:  # sideffected
-                    sideffected = name
-                    sfx_str = ", ".join(repr(i) for i in sideffects)
-
-                    ## Repr display also optionality & mapped-fn-kw
-                    #  (irrelevant to object's identity)
-                    #
-                    qmark = "?" if optional else ""
-                    # Mapped string is so convoluted bc it mimics `optional`
-                    # when `fn_kwarg` given.
-                    map_str = (
-                        f", fn_kwarg={fn_kwarg!r}"
-                        if fn_kwarg and fn_kwarg != name
-                        else ""
-                    )
-                    _repr = f"sideffected{qmark}({str(name)!r}<--{sfx_str}{map_str})"
-
-                    name = f"sideffected({str(name)!r}<--{sfx_str})"
-            elif optional or fn_kwarg:
-                map_str = f"-->{fn_kwarg!r}" if fn_kwarg != name else ""
-                _repr = (
-                    f"{'optional' if optional else 'mapped'}({str(name)!r}{map_str})"
-                )
-
-        obj = str.__new__(cls, name)
-
-        obj._repr = str(_repr) if _repr is not None else None
+        obj = super().__new__(cls, name)
         obj.fn_kwarg = fn_kwarg
         obj.optional = optional
         obj.sideffected = sideffected
-        obj.sideffects = sideffects
+        obj.sfx_list = sfx_list
+        obj._repr = _repr
 
         return obj
 
     def __repr__(self):
-        return super().__repr__() if self._repr is None else self._repr
+        return self._repr
 
-    def withset(self, **kw):
-        """
-        Make a new modifier with kwargs: name(or sideffected), fn_kwarg, optional, sideffects
-
-        :param optional:
-            either a bool or an :class:`_Optionals` enum, as taken from :attr:`.optional`
-            from another modifier instance
-        """
-        dep = _Modifier(
-            kw.pop("name", self.sideffected if self.sideffected else str(self)),
-            kw.pop("fn_kwarg", self.fn_kwarg),
-            kw.pop("optional", self.optional),
-            kw.pop("sideffects", self.sideffects),
+    def __getnewargs__(self):
+        return (
+            str(self),
+            self.fn_kwarg,
+            self.optional,
+            self.sideffected,
+            self.sfx_list,
+            self._repr,
         )
-        if kw:
-            raise ValueError(
-                f"Invalid kwargs: {kw}"
-                "\n  valid kwargs: name(or sideffected), fn_kwarg, optional, sideffects"
+
+    def _withset(
+        self,
+        name=...,
+        fn_kwarg=...,
+        optional: _Optionals = ...,
+        sideffected=...,
+        sfx_list=...,
+    ) -> Union["_Modifier", str]:
+        """
+        Make a new modifier with changes -- handle with care.
+
+        :return:
+             Delegates to :func:`_dep`, so returns a plain string if no args left.
+        """
+        kw = {
+            k: getattr(self, k) if v is ... else v
+            for k, v in zip(
+                "fn_kwarg optional sideffected sfx_list".split(),
+                (fn_kwarg, optional, sideffected, sfx_list),
             )
-        return dep
+        }
+        if name is ...:
+            name = self.sideffected or str(self)
+
+        return _dep(name=name, **kw)
 
 
-def is_mapped(dep) -> Optional[str]:
+def _dep(
+    name, fn_kwarg=None, optional: _Optionals = None, sideffected=None, sfx_list=(),
+):
     """
-    Check if a :term:`dependency` is mapped (and get it).
+    A :class:`_Modifier` factory that may return a plain str when no other args given.
 
-    Note that all non-varargish optionals are mapped (including sideffected optionals).
+    It decides the final `name` and `_repr` for the new modifier by matching
+    the given inputs with the :data:`_modifier_cstor_matrix`.
     """
-    return getattr(dep, "fn_kwarg", None)
+    formats = _match_modifier_args(name, fn_kwarg, optional, sideffected, sfx_list,)
+    if not formats:
+        # Make a plain string instead.
+        return str(name)
+
+    str_fmt, repr_fmt = formats
+    fmt_args = {
+        "dep": name,
+        "kw": f">'{fn_kwarg}'" if fn_kwarg != name else ">" if not optional else "",
+        "sfx": ", ".join(f"'{i}'" for i in sfx_list),
+    }
+    name = str_fmt % fmt_args
+    _repr = repr_fmt % fmt_args
+
+    return _Modifier(name, fn_kwarg, optional, sideffected, sfx_list, _repr)
 
 
-def is_optional(dep) -> bool:
-    """Check (and get) if a :term:`dependency` is optional (varargish/sideffects included)."""
-    return getattr(dep, "optional", None)
-
-
-def is_vararg(dep) -> bool:
-    """Check if an :term:`optionals` dependency is `vararg`."""
-    return getattr(dep, "optional", None) is _Optionals.vararg
-
-
-def is_varargs(dep) -> bool:
-    """Check if an :term:`optionals` dependency is `varargs`."""
-    return getattr(dep, "optional", None) is _Optionals.varargs
-
-
-def is_varargish(dep) -> bool:
-    """Check if an :term:`optionals` dependency is :term:`varargish`."""
-    try:
-        return dep.optional.varargish
-    except Exception:
-        return False
-
-
-def is_sideffect(dep) -> bool:
-    """Check if a dependency is :term:`sideffects` or :term:`sideffected`."""
-    return getattr(dep, "sideffects", None) is not None
-
-
-def is_pure_sideffect(dep) -> bool:
-    """Check if it is :term:`sideffects` but not a :term:`sideffected`."""
-    return getattr(dep, "sideffects", None) == ()
-
-
-def is_sideffected(dep) -> bool:
-    """Check if it is :term:`sideffected` (and get it)."""
-    return getattr(dep, "sideffected", None)
-
-
-def rename_dependency(dep, ren):
-    """Renames based to a fixed string or calling `ren` if callable, mapped to old"""
-    if callable(ren):
-        renamer = ren
-    else:
-        renamer = lambda n: ren
-
-    if isinstance(dep, _Modifier):
-        if is_pure_sideffect(dep):
-            pass  # TODO: rename sfx?
-        else:
-            old_name = dep.sideffected if is_sideffected(dep) else str(dep)
-            new_name = renamer(old_name)
-            dep = dep.withset(name=new_name)
-    else:  # plain string
-        dep = renamer(dep)
-
-    return dep
-
-
-def mapped(name: str, fn_kwarg: str):
+def mapped(name: str, fn_kwarg: str = None):
     """
     Annotate a :term:`needs` that (optionally) map `inputs` name --> argument-name.
 
@@ -219,7 +205,8 @@ def mapped(name: str, fn_kwarg: str):
     :param fn_kwarg:
         The argument-name corresponding to this named-input.
         If it is None, assumed the same as `name`, so as
-        to behave always like kw-type arg and to preserve fn-name if ever renamed.
+        to behave always like kw-type arg, and to preserve its fn-name
+        if ever renamed.
 
         .. Note::
             This extra mapping argument is needed either for :term:`optionals`
@@ -243,7 +230,7 @@ def mapped(name: str, fn_kwarg: str):
         ...    return a + b
         >>> myadd
         FunctionalOperation(name='myadd',
-                            needs=['a', mapped('name-in-inputs'-->'b')],
+                            needs=['a', 'name-in-inputs'(>'b')],
                             provides=['sum'],
                             fn='myadd')
 
@@ -257,7 +244,8 @@ def mapped(name: str, fn_kwarg: str):
 
         .. graphtik::
     """
-    return _Modifier(name, fn_kwarg=fn_kwarg or name)
+    # Must pass a truthy `fn_kwarg` bc cstor cannot not know its mapped.
+    return _dep(name, fn_kwarg=fn_kwarg or name)
 
 
 def optional(name: str, fn_kwarg: str = None):
@@ -272,7 +260,8 @@ def optional(name: str, fn_kwarg: str = None):
     :param fn_kwarg:
         the name for the function argument it corresponds;
         if a falsy is given, same as `name` assumed,
-        to behave always like kw-type arg and to preserve fn-name if ever renamed.
+        to behave always like kw-type arg and to preserve its fn-name
+        if ever renamed.
 
     **Example:**
 
@@ -289,7 +278,7 @@ def optional(name: str, fn_kwarg: str = None):
         >>> graph = compose('mygraph', myadd)
         >>> graph
         NetworkOperation('mygraph',
-                         needs=['a', optional('b')],
+                         needs=['a', 'b'(?)],
                          provides=['sum'],
                          x1 ops: myadd)
 
@@ -308,12 +297,13 @@ def optional(name: str, fn_kwarg: str = None):
         ...           provides="sum"
         ... )(myadd.fn)  # Cannot wrap an operation, its `fn` only.
         FunctionalOperation(name='myadd',
-                            needs=['a', optional('quasi-real'-->'b')],
+                            needs=['a', 'quasi-real'(?>'b')],
                             provides=['sum'],
                             fn='myadd')
 
     """
-    return _Modifier(name, fn_kwarg=fn_kwarg or name, optional=_Optionals.optional)
+    # Must pass a truthy `fn_kwarg` as cstor-matrix requires.
+    return _dep(name, fn_kwarg=fn_kwarg or name, optional=_Optionals.keyword)
 
 
 def vararg(name: str):
@@ -337,7 +327,10 @@ def vararg(name: str):
         ... def addall(a, *b):
         ...    return a + sum(b)
         >>> addall
-        FunctionalOperation(name='addall', needs=['a', vararg('b'), vararg('c')], provides=['sum'], fn='addall')
+        FunctionalOperation(name='addall',
+                            needs=['a', 'b'(*), 'c'(*)],
+                            provides=['sum'],
+                            fn='addall')
 
 
         >>> graph = compose('mygraph', addall)
@@ -354,7 +347,7 @@ def vararg(name: str):
         {'a': 5, 'sum': 5}
 
     """
-    return _Modifier(name, optional=_Optionals.vararg)
+    return _dep(name, optional=_Optionals.vararg)
 
 
 def varargs(name: str):
@@ -378,7 +371,7 @@ def varargs(name: str):
         ... )
         >>> graph
         NetworkOperation('mygraph',
-                         needs=['a', optional('b')],
+                         needs=['a', 'b'(?)],
                          provides=['sum'],
                          x1 ops: enlist)
 
@@ -394,9 +387,9 @@ def varargs(name: str):
         Traceback (most recent call last):
         ...
         graphtik.base.MultiValueError: Failed preparing needs:
-            1. Expected needs[varargs('b')] to be non-str iterables!
+            1. Expected needs['b'(+)] to be non-str iterables!
             +++inputs: ['a', 'b']
-            +++FunctionalOperation(name='enlist', needs=['a', varargs('b')], provides=['sum'], fn='enlist')
+            +++FunctionalOperation(name='enlist', needs=['a', 'b'(+)], provides=['sum'], fn='enlist')
 
     .. varargs-mistake-start
     .. Attention::
@@ -407,19 +400,19 @@ def varargs(name: str):
         Traceback (most recent call last):
         ...
         graphtik.base.MultiValueError: Failed preparing needs:
-            1. Expected needs[varargs('b')] to be non-str iterables!
+            1. Expected needs['b'(+)] to be non-str iterables!
             +++inputs: ['a', 'b']
             +++FunctionalOperation(name='enlist',
-                                   needs=['a', varargs('b')],
+                                   needs=['a', 'b'(+)],
                                    provides=['sum'],
                                    fn='enlist')
 
     .. varargs-mistake-end
     """
-    return _Modifier(name, optional=_Optionals.varargs)
+    return _dep(name, optional=_Optionals.varargs)
 
 
-def sideffect(name, optional: bool = None):
+def sfx(name, optional: bool = None):
     """
     :term:`sideffects` denoting modifications beyond the scope of the solution.
 
@@ -437,17 +430,17 @@ def sideffect(name, optional: bool = None):
       as *canceled* by returning it with a falsy value (operation must `returns dictionary`).
 
     .. hint::
-        If modifications involve some input/output, prefer the :func:`.sideffected`
+        If modifications involve some input/output, prefer the :func:`.sfxed`
         modifier.
 
         You may still convey this relationships by including the dependency name
         in the string - in the end, it's just a string - but no enforcement of any kind
         will happen from *graphtik*, like:
 
-        >>> from graphtik import sideffect
+        >>> from graphtik import sfx
 
-        >>> sideffect("price[sales_df]")
-        sideffect: 'price[sales_df]'
+        >>> sfx("price[sales_df]")
+        sfx: 'price[sales_df]'
 
     **Example:**
 
@@ -455,9 +448,9 @@ def sideffect(name, optional: bool = None):
     outside `solution`:
 
 
-        >>> from graphtik import operation, compose, sideffect
+        >>> from graphtik import operation, compose, sfx
 
-        >>> @operation(provides=sideffect("lights off"))  # sideffect names can be anything
+        >>> @operation(provides=sfx("lights off"))  # sideffect names can be anything
         ... def close_the_lights():
         ...    pass
 
@@ -465,12 +458,13 @@ def sideffect(name, optional: bool = None):
         ...     close_the_lights,
         ...     operation(
         ...         name='undress',
-        ...         needs=[sideffect("lights off")],
+        ...         needs=[sfx("lights off")],
         ...         provides="body")(lambda: "TaDa!")
         ... )
         >>> graph
-        NetworkOperation('strip ease', needs=[sideffect: 'lights off'],
-                         provides=[sideffect: 'lights off', 'body'],
+        NetworkOperation('strip ease',
+                         needs=[sfx: 'lights off'],
+                         provides=[sfx: 'lights off', 'body'],
                          x2 ops: close_the_lights, undress)
 
         >>> sol = graph()
@@ -483,35 +477,24 @@ def sideffect(name, optional: bool = None):
     .. note::
         Something has to provide a sideffect for a function needing it to execute -
         this could be another operation, like above, or the user-inputs;
-        just specify some dummy value for the sideffect:
+        just specify some truthy value for the sideffect:
 
-            >>> sol = graph.compute({sideffect("lights off"): True})
+            >>> sol = graph.compute({sfx("lights off"): True})
 
         .. graphtik::
 
     """
-    if type(name) is not str:
-        # import re
-        # if is_pure_sideffect(name):
-        #     m = re.match(r"sideffect\((.*)\)", name)
-        #     if m:
-        #         name = m.group(1)
-        # else:
-        raise ValueError(
-            "Expecting a regular string for sideffect"
-            f", got: {type(name).__name__}({name!r})"
-        )
-    return _Modifier(
-        name, optional=_Optionals.optional if optional else None, sideffects=()
+    return _dep(
+        name, optional=_Optionals.keyword if optional else None, sideffected=name,
     )
 
 
-def sideffected(
+def sfxed(
     dependency: str,
-    sideffect0: str,
-    *sideffects: str,
-    optional: bool = None,
+    sfx0: str,
+    *sfx_list: str,
     fn_kwarg: str = None,
+    optional: bool = None,
 ):
     r"""
     Annotates a :term:`sideffected` dependency in the solution sustaining side-effects.
@@ -519,10 +502,10 @@ def sideffected(
     :param fn_kwarg:
         the name for the function argument it corresponds.
         When optional, it becomes the same as `name` if falsy, so as
-        to behave always like kw-type arg and to preserve fn-name if ever renamed.
+        to behave always like kw-type arg, and to preserve fn-name if ever renamed.
         When not optional, if not given, it's all fine.
-        
-    Like :func:`.sideffect` but annotating a *real* :term:`dependency` in the solution,
+
+    Like :func:`.sfx` but annotating a *real* :term:`dependency` in the solution,
     allowing that dependency to be present both in :term:`needs` and :term:`provides`
     of the same function.
 
@@ -532,10 +515,10 @@ def sideffected(
     pandas dataframes (:gray:`emulated with dictionaries`):
 
 
-        >>> from graphtik import operation, compose, sideffected
+        >>> from graphtik import operation, compose, sfxed
 
         >>> @operation(needs="order_items",
-        ...            provides=sideffected("ORDER", "Items", "Prices"))
+        ...            provides=sfxed("ORDER", "Items", "Prices"))
         ... def new_order(items: list) -> "pd.DataFrame":
         ...     order = {"items": items}
         ...     # Pretend we get the prices from sales.
@@ -543,16 +526,16 @@ def sideffected(
         ...     return order
 
         >>> @operation(
-        ...     needs=[sideffected("ORDER", "Items"), "vat rate"],
-        ...     provides=sideffected("ORDER", "VAT")
+        ...     needs=[sfxed("ORDER", "Items"), "vat rate"],
+        ...     provides=sfxed("ORDER", "VAT")
         ... )
         ... def fill_in_vat(order: "pd.DataFrame", vat: float):
         ...     order['VAT'] = [i * vat for i in order['prices']]
         ...     return order
 
         >>> @operation(
-        ...     needs=[sideffected("ORDER", "Prices", "VAT")],
-        ...     provides=sideffected("ORDER", "Totals")
+        ...     needs=[sfxed("ORDER", "Prices", "VAT")],
+        ...     provides=sfxed("ORDER", "Totals")
         ... )
         ... def finalize_prices(order: "pd.DataFrame"):
         ...     order['totals'] = [p + v for p, v in zip(order['prices'], order['VAT'])]
@@ -566,16 +549,15 @@ def sideffected(
         >>> with debug_enabled(True):
         ...     finalize_prices
         FunctionalOperation(name='finalize_prices',
-                            needs=[sideffected('ORDER'<--'Prices'),
-                                   sideffected('ORDER'<--'VAT')],
-                            op_needs=[sideffected('ORDER'<--'Prices'),
-                                      sideffected('ORDER'<--'VAT')],
+                            needs=[sfxed('ORDER', 'Prices'), sfxed('ORDER', 'VAT')],
+                            op_needs=[sfxed('ORDER', 'Prices'), sfxed('ORDER', 'VAT')],
                             fn_needs=['ORDER'],
-                            provides=[sideffected('ORDER'<--'Totals')],
-                            op_provides=[sideffected('ORDER'<--'Totals')],
-                            fn_provides=['ORDER'], fn='finalize_prices')
+                            provides=[sfxed('ORDER', 'Totals')],
+                            op_provides=[sfxed('ORDER', 'Totals')],
+                            fn_provides=['ORDER'],
+                            fn='finalize_prices')
 
-    Notice that declaring a single *sideffected* with multiple *sideffects*,
+    Notice that declaring a single *sideffected* with many items in `sfx_list`,
     expands into multiple  *"singular"* ``sideffected`` dependencies in the network
     (check ``needs`` & ``op_needs`` above).
 
@@ -604,25 +586,130 @@ def sideffected(
 
 
     """
-    sideffects = (sideffect0,) + sideffects
-    ## Sanity checks
-    #
-    invalids = [f"{type(i).__name__}({i!r})" for i in sideffects if type(i) is not str]
-    if invalids:
-        raise ValueError(f"Expecting regular strings as sideffects, got: {invalids!r}")
-    if is_sideffect(dependency):
-        raise ValueError(
-            f"Expecting a non-sideffect for sideffected"
-            f", got: {type(dependency).__name__}({dependency!r})"
-        )
-    ## Mimic `optional` behavior,
-    #  i.e. preserve kwarg-ness if optional.
-    #
-    if optional and not fn_kwarg:
-        fn_kwarg = dependency
-    return _Modifier(
+    return _dep(
         dependency,
-        sideffects=sideffects,
-        optional=_Optionals.optional if optional else None,
-        fn_kwarg=fn_kwarg,
+        optional=_Optionals.keyword if optional else None,
+        fn_kwarg=dependency if optional and not fn_kwarg else fn_kwarg,
+        sideffected=dependency,
+        sfx_list=(sfx0, *sfx_list),
     )
+
+
+def sfxed_vararg(dependency: str, sfx0: str, *sfx_list: str):
+    """Like :func:`sideffected` + :func:`vararg`. """
+    return _dep(
+        dependency,
+        optional=_Optionals.vararg,
+        sideffected=dependency,
+        sfx_list=(sfx0, *sfx_list),
+    )
+
+
+def sfxed_varargs(dependency: str, sfx0: str, *sfx_list: str):
+    """Like :func:`sideffected` + :func:`varargs`. """
+    return _dep(
+        dependency,
+        optional=_Optionals.varargs,
+        sideffected=dependency,
+        sfx_list=(sfx0, *sfx_list),
+    )
+
+
+def is_mapped(dep) -> Optional[str]:
+    """
+    Check if a :term:`dependency` is mapped (and get it).
+
+    All non-varargish optionals are "mapped" (including sideffected ones).
+
+    :return:
+        the :attr:`fn_kwarg`
+    """
+    return getattr(dep, "fn_kwarg", None)
+
+
+def is_optional(dep) -> bool:
+    """
+    Check if a :term:`dependency` is optional.
+
+    Varargish & optional sideffects are included.
+
+    :return:
+        the :attr:`optional`
+    """
+    return getattr(dep, "optional", None)
+
+
+def is_vararg(dep) -> bool:
+    """Check if an :term:`optionals` dependency is `vararg`."""
+    return getattr(dep, "optional", None) == _Optionals.vararg
+
+
+def is_varargs(dep) -> bool:
+    """Check if an :term:`optionals` dependency is `varargs`."""
+    return getattr(dep, "optional", None) == _Optionals.varargs
+
+
+def is_varargish(dep) -> bool:
+    """Check if an :term:`optionals` dependency is :term:`varargish`."""
+    return dep.optional in (_Optionals.vararg, _Optionals.vararg)
+
+
+def is_sfx(dep) -> bool:
+    """
+    Check if a dependency is :term:`sideffects` or :term:`sideffected`.
+
+    :return:
+        the :attr:`sideffected`
+    """
+    return getattr(dep, "sideffected", None)
+
+
+def is_pure_sfx(dep) -> bool:
+    """Check if it is :term:`sideffects` but not a :term:`sideffected`."""
+    return getattr(dep, "sideffected", None) and not getattr(dep, "sfx_list", None)
+
+
+def is_sfxed(dep) -> bool:
+    """Check if it is :term:`sideffected`."""
+    return getattr(dep, "sideffected", None) and getattr(dep, "sfx_list", None)
+
+
+def dep_renamed(dep, ren):
+    """
+    Renames `dep` as `ren` or call `ren`` (if callable) to decide its name,
+
+    preserving any :func:`mapped` to old-name.
+    """
+    if callable(ren):
+        renamer = ren
+    else:
+        renamer = lambda n: ren
+
+    if isinstance(dep, _Modifier):
+        old_name = dep.sideffected if is_sfx(dep) else str(dep)
+        new_name = renamer(old_name)
+        dep = dep._withset(name=new_name)
+    else:  # plain string
+        dep = renamer(dep)
+
+    return dep
+
+
+def dep_singularized(dep):
+    """
+    Yield one sideffected for each sfx in :attr:`.sfx_list`, or iterate `dep` in other cases.
+    """
+    return (
+        (dep._withset(sfx_list=(s,)) for s in dep.sfx_list) if is_sfxed(dep) else (dep,)
+    )
+
+
+def dep_stripped(dep):
+    """
+    Return the :attr:`_Modifier.sideffected` if `dep` is :term:`sideffected`, `dep` otherwise,
+
+    conveying all other properties of the original modifier to the stripped dependency.
+    """
+    if is_sfxed(dep):
+        dep = dep._withset(name=dep.sideffected, sideffected=None, sfx_list=())
+    return dep
