@@ -24,7 +24,7 @@ from boltons.setutils import IndexedSet as iset
 from .base import Items, PlotArgs, Plottable, astuple, jetsam
 from .config import is_debug, is_skip_evictions
 from .modifiers import dep_renamed, is_mapped, is_optional, is_sfx, optional
-from .op import FunctionalOperation, Operation
+from .composition import FunctionalOperation, Operation
 
 NodePredicate = Callable[[Any, Mapping], bool]
 
@@ -639,8 +639,7 @@ def build_network(
         see same-named param in :func:`.compose`
     """
     from boltons.setutils import IndexedSet as iset
-    from .op import NULL_OP
-    from .pipeline import NetworkOperation
+    from .composition import NULL_OP, NetworkOperation
 
     def proc_op(op, parent=None):
         """clone FuncOperation with certain props changed"""
